@@ -22,7 +22,6 @@
 This module provides an OpenStreetMap downloader.
 """
 
-import collections
 import sys
 import math
 import urllib.request
@@ -142,10 +141,6 @@ class OpenStreetMap:
         z = min(max(z, self.z_min), self.z_max)
         ntiles = 2**z
 
-        # do we need it?
-        # if ntiles * 256 < self.height:
-        #     lat = 0
-
         # calculate x,y offsets to lat,lon within width,height
         xc, yc = self.latlon_to_tile(lat, lon, z)
         x, y = int(xc), int(yc)
@@ -209,7 +204,6 @@ class OpenStreetMap:
                     cached_tile.raw = False
                 self.draw_tile(image, cached_tile.image, tile.x, tile.y)
         self.draw_image(image)
-        # try to not clear self.rescaled_tiles to keep rescale() safer
         self.rescaled_tiles = self.tiles.copy()
 
     def draw_rescaled(self):
@@ -255,19 +249,22 @@ class OpenStreetMap:
         self.dz += dz
         if ((self.z < self.z_max and self.dz >= 1) or
             (self.z > self.z_min and self.dz <= -1)):
-            dz = 1 if self.dz > 0 else -1
+            dz = int(self.dz)
+            self.message("rescale:", self.z, dz)
             z = self.z + dz
+
             # pinned zoom at x,y
-            if dz > 0:
-                # each zoom-in doubles
-                xc = (x + self.width / 2) / 2
-                yc = (y + self.height / 2) / 2
-                self.message("zoom in:", z)
-            else:
-                # each zoom-out halves
-                xc = self.width - x
-                yc = self.height - y
-                self.message("zoom out:", z)
+            xc, yc = self.width / 2, self.height / 2
+            for i in range(0, abs(dz)):
+                if dz > 0:
+                    # each zoom-in doubles
+                    xc = (x + xc) / 2
+                    yc = (y + yc) / 2
+                else:
+                    # each zoom-out halves
+                    xc = 2 * xc - x
+                    yc = 2 * yc - y
+
             # lat,lon at xc,yc
             lat, lon = self.canvas_to_latlon(xc, yc)
             zoomed = True
@@ -337,8 +334,11 @@ class OpenStreetMap:
         self.dz += dz
         if ((self.z < self.z_max and self.dz >= 1) or
             (self.z > self.z_min and self.dz <= -1)):
-            dz = 1 if self.dz > 0 else -1
+            dz = int(self.dz)
+            self.message("rescale:", self.z, dz)
             z = self.z + dz
+
+            # pinned rescale at x,y
             xc, yc = self.width / 2, self.height / 2
             for i in range(0, abs(dz)):
                 if dz > 0:
@@ -396,7 +396,7 @@ class OpenStreetMap:
     def repeat_xy(self, xy):
         outxy = []
         n = self.width // (256 * self.ntiles)
-        for i in range(-n//2-1, n//2+2):
+        for i in range(-n // 2 - 1, n // 2 + 2):
             dx = i * 256 * self.ntiles
             p = []
             for coor in xy:
